@@ -79,6 +79,11 @@ module Excon
         params[:path].insert(0, '/')
       end
 
+      if block_given?
+        puts("Excon requests with a block are deprecated, pass :response_block instead (#{caller.first})")
+        params[:response_block] = Proc.new
+      end
+
       if params.has_key?(:instrumentor)
         if (retries_remaining ||= params[:retry_limit]) < params[:retry_limit]
           event_name = "#{params[:instrumentor_name]}.retry"
@@ -86,10 +91,10 @@ module Excon
           event_name = "#{params[:instrumentor_name]}.request"
         end
         params[:instrumentor].instrument(event_name, params) do
-          request_kernel(params, &block)
+          request_kernel(params)
         end
       else
-        request_kernel(params, &block)
+        request_kernel(params)
       end
     rescue => request_error
       if params[:idempotent] && [Excon::Errors::SocketError,
@@ -140,10 +145,28 @@ module Excon
 
   private
 
-    def request_kernel(params, &block)
+    def detect_content_length(body)
+      if body.is_a?(String)
+        if FORCE_ENC
+          body.force_encoding('BINARY')
+        end
+        body.length
+      elsif body.respond_to?(:size)
+        # IO object: File, Tempfile, etc.
+        body.size
+      else
+        begin
+          File.size(body) # for 1.8.7 where file does not have size
+        rescue
+          0
+        end
+      end
+    end
+
+    def request_kernel(params)
       begin
         response = if params[:mock]
-          invoke_stub(params, &block)
+          invoke_stub(params)
         else
           socket.params = params
           # start with "METHOD /path"
@@ -174,24 +197,12 @@ module Excon
           # finish first line with "HTTP/1.1\r\n"
           request << HTTP_1_1
 
-          # calculate content length and set to handle non-ascii
-          unless params[:headers].has_key?('Content-Length')
+          if params.has_key?(:request_block)
+            params[:headers]['Transfer-Encoding'] = 'chunked'
+          elsif ! (params[:method].to_s.casecmp('GET') == 0 && params[:body].nil?)
             # The HTTP spec isn't clear on it, but specifically, GET requests don't usually send bodies;
             # if they don't, sending Content-Length:0 can cause issues.
-            unless (params[:method].to_s.casecmp('GET') == 0 && params[:body].nil?)
-              params[:headers]['Content-Length'] = case params[:body]
-              when File
-                params[:body].binmode
-                File.size(params[:body])
-              when String
-                if FORCE_ENC
-                  params[:body].force_encoding('BINARY')
-                end
-                params[:body].length
-              else
-                0
-              end
-            end
+            params[:headers]['Content-Length'] = detect_content_length(params[:body])
           end
 
           # add headers to request
@@ -208,10 +219,26 @@ module Excon
           socket.write(request)
 
           # write out the body
-          unless params[:body].nil? || params[:body].empty?
+          if params.has_key?(:request_block)
+            while true
+              chunk = params[:request_block].call
+              if FORCE_ENC
+                chunk.force_encoding('BINARY')
+              end
+              if chunk.length > 0
+                socket.write(chunk.length.to_s(16) << CR_NL << chunk << CR_NL)
+              else
+                socket.write('0' << CR_NL << CR_NL)
+                break
+              end
+            end
+          elsif !params[:body].nil?
             if params[:body].is_a?(String)
-              socket.write(params[:body])
+              unless params[:body].empty?
+                socket.write(params[:body])
+              end
             else
+              params[:body].binmode if params[:body].respond_to?(:binmode)
               while chunk = params[:body].read(CHUNK_SIZE)
                 socket.write(chunk)
               end
@@ -219,7 +246,7 @@ module Excon
           end
 
           # read the response
-          response = Excon::Response.parse(socket, params, &block)
+          response = Excon::Response.parse(socket, params)
 
           if response.headers['Connection'] == 'close'
             reset
@@ -243,7 +270,6 @@ module Excon
     end
 
     def invoke_stub(params)
-      block_given = block_given?
       params[:captures] = {:headers => {}} # setup data to hold captures
       for stub, response in Excon.stubs
         headers_match = !stub.has_key?(:headers) || stub[:headers].keys.all? do |key|
@@ -276,17 +302,14 @@ module Excon
             response
           end
 
-          # don't pass stuff into a block if there was an error
           if params[:expects] && ![*params[:expects]].include?(response_attributes[:status])
-            block_given = false
-          end
-
-          if block_given && response_attributes.has_key?(:body)
+            # don't pass stuff into a block if there was an error
+          elsif params.has_key?(:response_block) && response_attributes.has_key?(:body)
             body = response_attributes.delete(:body)
             content_length = remaining = body.bytesize
             i = 0
             while i < body.length
-              yield(body[i, CHUNK_SIZE], [remaining - CHUNK_SIZE, 0].max, content_length)
+              params[:response_block].call(body[i, CHUNK_SIZE], [remaining - CHUNK_SIZE, 0].max, content_length)
               remaining -= CHUNK_SIZE
               i += CHUNK_SIZE
             end
