@@ -1,6 +1,6 @@
 module Excon
   unless const_defined?(:VERSION)
-    VERSION = '0.10.0'
+    VERSION = '0.13.4'
   end
 
   unless const_defined?(:CHUNK_SIZE)
@@ -11,6 +11,10 @@ module Excon
     CR_NL = "\r\n"
   end
 
+  unless const_defined?(:DEFAULT_CA_FILE)
+    DEFAULT_CA_FILE = File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "data", "cacert.pem"))
+  end
+
   unless const_defined?(:DEFAULT_RETRY_LIMIT)
     DEFAULT_RETRY_LIMIT = 4
   end
