@@ -1,7 +1,7 @@
 module Excon
   class SSLSocket < Socket
 
-    # backwards compatability for 1.8.x SSLSocket (which lack nonblock)
+    # backwards compatability for things lacking connect_nonblock
     unless OpenSSL::SSL::SSLSocket.public_method_defined?(:connect_nonblock)
 
       undef_method :connect
@@ -9,9 +9,19 @@ module Excon
         @socket = TCPSocket.new(@params[:host], @params[:port])
       end
 
+    end
+
+    # backwards compatability for things lacking read_nonblock
+    unless OpenSSL::SSL::SSLSocket.public_method_defined?(:read_nonblock)
+
       undef_method :read
       def_delegators(:@socket, :read, :read)
 
+    end
+
+    # backwards compatability for things lacking write_nonblock
+    unless OpenSSL::SSL::SSLSocket.public_method_defined?(:write_nonblock)
+
       undef_method :write
       def_delegators(:@socket, :write, :write)
 
@@ -23,14 +33,14 @@ module Excon
       # create ssl context
       ssl_context = OpenSSL::SSL::SSLContext.new
 
-      if Excon.defaults[:ssl_verify_peer]
+      if params[:ssl_verify_peer]
         # turn verification on
         ssl_context.verify_mode = OpenSSL::SSL::VERIFY_PEER
 
-        if Excon.defaults[:ssl_ca_path]
-          ssl_context.ca_path = Excon.defaults[:ssl_ca_path]
-        elsif Excon.defaults[:ssl_ca_file]
-          ssl_context.ca_file = Excon.defaults[:ssl_ca_file]
+        if params[:ssl_ca_path]
+          ssl_context.ca_path = params[:ssl_ca_path]
+        elsif params[:ssl_ca_file]
+          ssl_context.ca_file = params[:ssl_ca_file]
         else
           # use default cert store
           store = OpenSSL::X509::Store.new
@@ -64,7 +74,7 @@ module Excon
       @socket.connect
 
       # verify connection
-      if Excon.defaults[:ssl_verify_peer]
+      if params[:ssl_verify_peer]
         @socket.post_connection_check(@params[:host])
       end
 
